/*-- Copyright (c) 2011 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved. --*/
/*
Model         	DDCF FM Integration  
Subject Area  	Financial Management Integration
Version
Name          	DDCF 5.3: Macro to initiate consensus process
DBMS          	SAS
Created by 		Abhijeet Murumkar and kanchan Kamat
Created on 		17/06/2011 14:45:00 PM
Last updated 	20/09/2011 18:15:00 PM

* Copyright (c) 2011 by SAS Institute Inc., Cary, NC 27513

Input Parameters 
1. sassdm		Library name of FM sdm tables
2. plan_id		Plan identification number

*/


/*options mlogic mprint symbolgen;*/
libname stagefm "&GL_FM_STAGE_PATH";

data _null_;
	call symputx('VarDtStr',TRANWRD(('_'|| put(date(),YYMMDDN.) || '_' || put(time(),HHMM.) || '.log'),':','_'));
run;

%let logfilenm=initiate_%sysfunc(ktrim(&plan_id.))&VarDtStr.;
%let logfilepath=&GL_DDF_CONFIG_DIR_PATH./logs/;

proc printto log="&logfilepath.&logfilenm."; 
run;	

%macro ddcf_fm_initiate_consensus_stp(sassdm=sassdm);

%if &GL_FRAMEWORK_DB_IND.=&GL_SHORT_NO. %then %do;
	libname uiart MYSQL  INSERTBUFF=&GL_MYSQL_INSERTBUFF. DBINDEX=YES bulkload = yes  MYSQL_PORT=3306 DBCONINIT = &GL_MYSQL_DBCONINIT.
	DATABASE=&GL_TRANS_DATABASE_NM. SERVER="&GL_TRANS_DB_SERVER."  authdomain = &GL_MYSQL_AUTH_DOMAIN. ; 
%end;


	libname sassdm MYSQL  INSERTBUFF=&GL_MYSQL_INSERTBUFF. DBINDEX=YES bulkload = yes  MYSQL_PORT=3306 DBCONINIT = &GL_MYSQL_DBCONINIT.
	DATABASE=&GL_FM_MYSQL_DATABASE_NM. SERVER="&GL_FM_MYSQL_SERVER."  authdomain = &GL_FM_MYSQL_AUTH_DOMAIN. ;


	/* Define connection string for transactional server*/
	%ddcf_connect_to_trans_db;

	%let job_rc=0;
	%let job_rc_msg=;

	proc options option=jreoptions; run;

	
	/*************** WORKFLOW ***************/

	/* Change Plan Status to promoted */

	proc sql noprint;
		update uiart.plan_details 
		set PLAN_STATUS_NO = 5 
		where plan_id = &plan_id.;
	quit;
	%RCSet(&sqlrc);

	/* WORKFLOW: DDCF CONSENSUS PLAN : To load plan specific dimension hierarchies and fact data to FM CDL */

	%if (&job_rc<5)%then %do;
		%ddcf_fm_stg_load_plan_data (plan_id = &plan_id. );
	%end;
	%else %do;
		%let consensus_status=7;
		%goto exit;
	%end;

	/* WORKFLOW: DDCF CONSENSUS PLAN : To load plan specific hierarchies */
	%if (&job_rc<5) %then %do;
		%ddcf_fm_sdm_load_dimensions(plan_id=&plan_id., load_members = "&GL_SHORT_NO.");
	
		%if (&job_rc<5) %then %do;
			/* 2	Load Dimensions Hierarchy data -Success */
			%let consensus_status=2;
			proc sql;
				update uiart.plan_consensus_details 
				set STATUS_CD = 2 
				where plan_id = &plan_id.;
			quit;
			%RCSet(&sqlrc);
		%end;
	%end;
	%else %do;
		/* 7	Load Dimensions Hierarchy data -Failed */
		%let consensus_status=7;
		%goto exit;
	%end;

	
	/* WORKFLOW: DDCF CONSENSUS PLAN : To check whether the cycle exist in FM SDM */

	%let cnt=0;
	/* Get schema id */
	%let consensus_status=0;
	%let schema_id=;
	%ddcf_get_schema_id(plan_id=&plan_id.);
	%put &schema_id.;

	proc sql noprint;
		select count(*) into :cnt
		from &sassdm..sas_cycle_data
		where ktrim(kleft(kupcase(CYCLE_NM))) = %sysfunc(kupcase(%sysfunc(ktrim(%sysfunc(kleft("&schema_id."))))));
	quit;
	%RCSet(&sqlrc);

	%if &cnt. = 0 %then %do;

		/* WORKFLOW: DDCF CONSENSUS PLAN : To create cycle */

		%if (&job_rc<5) %then %do;

			%ddcf_fm_create_cycle(plan_id=&plan_id.,sassdm=sassdm);

			%if (&job_rc<5) %then %do;
				/* 3	Create cycle -Success */
				%let consensus_status=3;
				proc sql;
					update uiart.plan_consensus_details 
					set STATUS_CD = 3 
					where plan_id = &plan_id.;
				quit;
				%RCSet(&sqlrc);
			%end;
		%end;
		%else %do;
			/* 8	Create cycle Success -Failed */
			%let consensus_status=8;
			%goto exit;
		%end;

		/* WORKFLOW: DDCF CONSENSUS PLAN : To create model */

		%if (&job_rc<5) %then %do;

			%ddcf_fm_create_model(plan_id=&plan_id.);

			%if (&job_rc<5) %then %do;
				/* 4	Create model-Success */
				%let consensus_status=4;
				proc sql;
					update uiart.plan_consensus_details 
					set STATUS_CD = 4 
					where plan_id = &plan_id.;
				quit;
				%RCSet(&sqlrc);
			%end;
		%end;
		%else %do;
			/* 9	Create model-Success -Failed */
			%let consensus_status=9;
			%goto exit;
		%end;

		/* WORKFLOW: DDCF CONSENSUS PLAN : To load fact data in a cycle  */

		%if (&job_rc<5) %then %do;

			%ddcf_fm_sdm_load_facts(plan_id=&plan_id.);

			%if (&job_rc<5) %then %do;
				/* 5	Load Fact data-Success */
				%let consensus_status=5;
				proc sql;
					update uiart.plan_consensus_details 
					set STATUS_CD = 5 
					where plan_id = &plan_id.;
				quit;
				%RCSet(&sqlrc);
			%end;
		%end;
		%else %do;
			/* 10	Load Fact data-Success -Failed */
			%let consensus_status=10;
			%goto exit;
		%end;

		/* WORKFLOW: DDCF CONSENSUS PLAN : To create phase */

		%if (&job_rc<5) %then %do;

			%ddcf_fm_create_phase(plan_id=&plan_id.);

			%if (&job_rc<5) %then %do;
				/* 6	Create Phase-Success */
				%let consensus_status=6;
				proc sql;
					update uiart.plan_consensus_details 
					set STATUS_CD = 6 
					where plan_id = &plan_id.;
				quit;
				%RCSet(&sqlrc);
			%end;
		%end;
		%else %do;
			/* 11	Create Phase-Success Failed */
			%let consensus_status=11;
			%goto exit;
		%end;

		/* WORKFLOW: DDCF CONSENSUS PLAN : To create Form Set manually  */

	%end;
	%else %do;

		/* WORKFLOW: DDCF CONSENSUS PLAN : To load additional time periods into cycle  */

		%if (&job_rc<5) %then %do;

			%ddcf_fm_update_cycle(plan_id=&plan_id.,sassdm=sassdm);

		%end;

		/* Reset Formsets */

		%if (&job_rc<5) %then %do;

			%ddcf_fm_reset_formset(plan_id=&plan_id.);

		%end;

		/* END: Reset Formsets */

		
		/* WORKFLOW: DDCF CONSENSUS PLAN : To load fact data in a cycle  */

		%if (&job_rc<5) %then %do;

			%ddcf_fm_sdm_load_facts(plan_id=&plan_id.);

			%if (&job_rc<5) %then %do;
				/* 5	Load Fact data-Success */
				%let consensus_status=5;
				proc sql;
					update uiart.plan_consensus_details 
					set STATUS_CD = 6 
					where plan_id = &plan_id.;
				quit;
				%RCSet(&sqlrc);
			%end;
		%end;
		%else %do;
			/* 10	Load Fact data-Success -Failed */
			%let consensus_status=10;
			%goto exit;
		%end;

	%end;

%exit:
	%if (&job_rc>4) %then %do;
		proc sql noprint;
			update uiart.plan_consensus_details 
			set STATUS_CD = &consensus_status. 
			where plan_id = &plan_id.;
		quit;
		%RCSet(&sqlrc);
	%end;

	libname sassdm clear;

%if &GL_FRAMEWORK_DB_IND.=&GL_SHORT_NO. %then %do;
	libname uiart clear;
%end;

%mend;

%ddcf_fm_initiate_consensus_stp(sassdm=sassdm);
proc printto; 
run;
